#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_form.interfaces.form import IWidgetForm
from pyams_skin.interfaces import IInnerPage, IPageHeader
from pyams_skin.interfaces.configuration import IConfiguration
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces.site import ISiteRoot
from pyams_zmi.interfaces.menu import ISiteManagementMenu, IPropertiesMenu
from pyams_zmi.layer import IAdminLayer

# import packages
from pyams_form.form import AJAXEditForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import adapter_config
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyramid.decorator import reify
from pyramid.view import view_config
from z3c.form import field
from zope.interface import implementer, implementedBy, Interface

from pyams_zmi import _


@viewlet_config(name='properties.menu', layer=IAdminLayer, context=ISiteRoot, manager=ISiteManagementMenu,
                permission='system.view', weight=1)
@viewletmanager_config(name='properties.menu', layer=IAdminLayer, context=ISiteRoot, provides=IPropertiesMenu)
@implementer(IPropertiesMenu)
class PropertiesMenuItem(MenuItem):
    """Properties menu"""

    label = _("Properties")
    icon_class = 'fa fa-fw fa-twitch'
    url = '#properties.html'


@pagelet_config(name='properties.html', context=ISiteRoot, layer=IPyAMSLayer, permission='system.view')
@implementer(IWidgetForm, IInnerPage)
class PropertiesEditForm(AdminEditForm):
    """Properties edit form"""

    legend = _("Update main site properties")
    ajax_handler = 'edit_properties.json'
    edit_permission = 'system.manage'

    @reify
    def fields(self):

        def get_configuration_interface(context):
            for interface in implementedBy(context.__class__).interfaces():
                if issubclass(interface, IConfiguration):
                    return interface

        content = self.getContent()
        return field.Fields(get_configuration_interface(content)).omit('__name__')

    def getContent(self):
        return IConfiguration(self.context)


@view_config(name='edit_properties.json', context=ISiteRoot, request_type=IPyAMSLayer,
             permission='system.manage', renderer='json', xhr=True)
class PropertiesAJAXEditForm(AJAXEditForm, PropertiesEditForm):
    """Properties edit form, AJAX renderer"""

    def get_ajax_output(self, changes):
        if changes:
            return {'status': 'reload',
                    'location': '#properties.html',
                    'smallbox': self.request.localizer.translate(self.successMessage),
                    'smallbox_status': 'success'}
        else:
            return AJAXEditForm.get_ajax_output(self, changes)


@adapter_config(context=(Interface, IPyAMSLayer, PropertiesEditForm), provides=IPageHeader)
class PropertiesEditFormHeaderAdapter(DefaultPageHeaderAdapter):
    """Utilities header adapter"""

    @property
    def title(self):
        config = IConfiguration(self.context)
        return config.title

    icon_class = 'fa fa-fw fa-twitch'
